@@ -44,7 +44,10 @@ class TypedDictSorter:
                 return self._sort_topological()
             except CycleError as e:
                 for typed_dict in self._get(*e.args[-1]):
-                    self.logger.debug(f"Stringifying {typed_dict.name}")
+                    self.logger.debug(
+                        f"Stringifying {typed_dict.name}: unsortable children"
+                        f" {self.typed_dicts_map[typed_dict.name]}"
+                    )
                     typed_dict.stringify()
 
         self.logger.warn("Topological sort failed, stringifying TypedDicts")
